"""twspace_dl formatting helper submodule"""
import os
import re
from collections import defaultdict
from datetime import datetime


# mostly taken from ytarchive
class FormatInfo(dict):
    """
    Simple class to more easily keep track of what fields are available for
    file name formatting
    """

    DEFAULT_FNAME_FORMAT = "(%(creator_name)s)%(title)s-%(id)s"

    def __init__(self) -> None:
        dict.__init__(
            self,
            {
                "id": "",
                "url": "",
                "title": "",
                "creator_name": "",
                "creator_screen_name": "",
                "start_date": "",
            },
        )

    def set_info(self, metadata: dict) -> None:
        root = defaultdict(str, metadata["data"]["audioSpace"]["metadata"])
        creator_info = root["creator_results"]["result"]["legacy"]  # type: ignore
        self["id"] = root["rest_id"]
        self["url"] = "https://twitter.com/i/spaces/" + self["id"]
        self["title"] = root["title"]
        self["creator_name"] = creator_info["name"]  # type: ignore
        self["creator_screen_name"] = creator_info["screen_name"]  # type: ignore
        self["start_date"] = datetime.fromtimestamp(
            int(root["started_at"]) / 1000
        ).strftime("%Y-%m-%d")

    # https://gist.github.com/dbr/256270
    @staticmethod
    def sterilize_fn(value: str) -> str:
        """
        Takes a string and makes it into a valid filename.
        """

        # Treat extension seperatly
        value, extension = os.path.splitext(value)

        # Remove null byte
        value = value.replace("\0", "")

        # If the filename starts with a . prepend it with an underscore, so it
        # doesn't become hidden
        if value.startswith("."):
            value = "_" + value

        # platform.system docs say it could also return "Windows" or "Java".
        # Failsafe and use Windows sanitisation for Java, as it could be any
        # operating system.
        blacklist = r"\/:*?\"<>|"

        # Replace every blacklisted character with a underscore
        value = re.sub("[%s]" % re.escape(blacklist), "_", value)

        # Remove any trailing whitespace
        value = value.strip()

        # There are a bunch of filenames that are not allowed on Windows.
        # As with character blacklist, treat non Darwin/Linux platforms as Windows
        invalid_filenames = [
            "CON",
            "PRN",
            "AUX",
            "NUL",
            "COM1",
            "COM2",
            "COM3",
            "COM4",
            "COM5",
            "COM6",
            "COM7",
            "COM8",
            "COM9",
            "LPT1",
            "LPT2",
            "LPT3",
            "LPT4",
            "LPT5",
            "LPT6",
            "LPT7",
            "LPT8",
            "LPT9",
        ]
        if value in invalid_filenames:
            value = "_" + value

        return value + extension

    def format(self, format_str: str) -> str:
        actual_format_str = os.path.basename(format_str)
        abs_dir = os.path.dirname(format_str)
        basename = self.sterilize_fn(actual_format_str % self)
        return os.path.join(abs_dir, basename)



"Module providing login utilities for twspace_dl"
import re

import requests


def load_from_file(filename: str) -> str:
    """return auth_token from netscape cookie file"""
    try:
        token = re.findall(
            r"(?<=auth_token\s)\w{40}", open(filename, "r", encoding="utf-8").read()
        )[0]
    except IndexError as err:
        raise ValueError(
            (
                "Cookie file does not have auth_token.\n"
                "Please check if you were connected when creating it"
            )
        ) from err
    return token


def write_to_file(auth_token: str, filename: str) -> None:
    """Write cookie to file in a format recognizable by the module
    (`auth_token        {auth_token}`)"""
    with open(filename, "w", encoding="utf-8") as cookie_file:
        cookie_file.write("auth_token" + 8 * " " + auth_token)


class Login:
    def __init__(self, username, password, guest_token):
        self.username = username
        self.password = password
        self.guest_token = guest_token
        self.session = requests.Session()
        self.task_url = "https://twitter.com/i/api/1.1/onboarding/task.json"
        self.flow_token: str

    def login(self) -> str:
        request_flow = self.session.post(
            self.task_url,
            params={"flow_name": "login"},
            headers=self._headers,
            json=self._initial_params,
        )
        try:
            self.flow_token = request_flow.json()["flow_token"]
        except KeyError as err:
            raise RuntimeError(
                "Error while initiating parameters:", request_flow.json()
            ) from err

        # js instrumentation subtask
        request_flow = self.session.post(
            self.task_url, headers=self._headers, json=self._js_instrumentation_data
        )
        try:
            self.flow_token = request_flow.json()["flow_token"]
        except KeyError as err:
            raise RuntimeError(
                "Error while performing js instrumentation:", request_flow.json()
            ) from err

        # user identifier sso subtask
        request_flow = self.session.post(
            self.task_url, headers=self._headers, json=self._user_identifier_sso_data
        )
        try:
            self.flow_token = request_flow.json()["flow_token"]
        except KeyError as err:
            raise RuntimeError("Error identifying user:", request_flow.json()) from err

        # account duplication check
        request_flow = self.session.post(
            self.task_url, headers=self._headers, json=self._account_dup_check_data
        )
        try:
            self.flow_token = request_flow.json()["flow_token"]
        except KeyError as err:
            raise RuntimeError(
                "Error while checking account duplication:", request_flow.json()
            ) from err

        # enter password
        request_flow = self.session.post(
            self.task_url, headers=self._headers, json=self._enter_password_data
        )
        try:
            auth_token = str(request_flow.cookies["auth_token"])
        except KeyError as err:
            raise RuntimeError(
                "Error while while entering password:", request_flow.json()
            ) from err

        return auth_token

    @property
    def _headers(self):
        return {
            "authorization": (
                "Bearer "
                "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
                "=1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
            ),
            "x-guest-token": self.guest_token,
        }

    @property
    def _js_instrumentation_data(self) -> dict:
        return {
            "flow_token": self.flow_token,
            "subtask_inputs": [
                {
                    "subtask_id": "LoginJsInstrumentationSubtask",
                    "js_instrumentation": {
                        "response": (
                            '{"rf":{"a976808c0d7d2c9a6081e997a1114f952a'
                            'f0067dcb59267ac8d852ec0ef98fe6":-65,"a1442a'
                            "8ec6ecb5804b8f60864cc54ded1bc140c4adaef6ac8"
                            '642d181ba17e0fd":1,"a3d158c7a0003247a36ff48'
                            '60f096c8eeefb23608d983162f94344db0b0a1f84":-10'
                            ',"a8031adb0e4372374f15125f9a10e5b8017743895f49'
                            '5b43c1aa74839c9a5876":15},"s":"HrR8ThECGdeQ4Ug'
                            "aWLIMSqGx0fL_7FOf7KjX8tlWN6WBN6HVcojL3if3rN"
                            "bYDtDhDmwK1jxMViInpjc1hc-kOO5w6Ej7WxoqdmI0eTVj-"
                            "5iul5FMdGaGZUVuWtkq3A7A42Y5RAsgNwYtpVB44XifZ3W1f"
                            "MscefI8HovjFtWUm0caZkF6_Y_1iFr0FSWHgM95gx0pXkK"
                            "910VlKn0HqT8Dvo6ss7LMA5Cf-VS84q284Vsx6h3nqwT"
                            "gzo4Nx3V4d86VL45GqIzqbwKT0OMlM6DHKk2Pi8WxKZ"
                            "_QoHAMQI0AzBCJ6McdfjGf7lCjtLLRb4ClfZNTW0g"
                            'IX3dMSEj03mvOkgAAAX4abfqW"}'
                        ),
                        "link": "next_link",
                    },
                }
            ],
        }

    @property
    def _user_identifier_sso_data(self) -> dict:
        # assert self.flow_token[-1] == "1"
        return {
            "flow_token": self.flow_token,
            "subtask_inputs": [
                {
                    "subtask_id": "LoginEnterUserIdentifierSSOSubtask",
                    "settings_list": {
                        "setting_responses": [
                            {
                                "key": "user_identifier",
                                "response_data": {
                                    "text_data": {"result": self.username}
                                },
                            }
                        ],
                        "link": "next_link",
                    },
                }
            ],
        }

    @property
    def _account_dup_check_data(self) -> dict:
        # assert self.flow_token[-1] == "2"
        return {
            "flow_token": self.flow_token,
            "subtask_inputs": [
                {
                    "subtask_id": "AccountDuplicationCheck",
                    "check_logged_in_account": {
                        "link": "AccountDuplicationCheck_false"
                    },
                }
            ],
        }

    @property
    def _enter_password_data(self) -> dict:
        # assert self.flow_token[-1] == "6"
        return {
            "flow_token": self.flow_token,
            "subtask_inputs": [
                {
                    "subtask_id": "LoginEnterPassword",
                    "enter_password": {"password": self.password, "link": "next_link"},
                }
            ],
        }

    @property
    def _initial_params(self) -> dict:
        return {
            "input_flow_data": {
                "flow_context": {
                    "debug_overrides": {},
                    "start_location": {"location": "splash_screen"},
                }
            },
            "subtask_versions": {
                "contacts_live_sync_permission_prompt": 0,
                "email_verification": 1,
                "topics_selector": 1,
                "wait_spinner": 1,
                "cta": 4,
            },
        }



import json
import logging
import os
import re
import shutil
import subprocess
import tempfile

from functools import cached_property
from urllib.parse import urlparse

import requests

from .format_info import FormatInfo


class TwspaceDL:
    """Downloader class for twitter spaces"""

    def __init__(self, space_id: str, format_str: str) -> None:
        self.id = space_id
        self.format_str = format_str or FormatInfo.DEFAULT_FNAME_FORMAT
        self.session = requests.Session()
        self._tmpdir: str

    @classmethod
    def from_space_url(cls, url: str, format_str: str):
        """Create a TwspaceDL object from a space url"""
        if not url:
            logging.warning("No space url given, file won't have any metadata")
            space_id = "no_id"
            format_str = "no_info"
        else:
            try:
                space_id = re.findall(r"(?<=spaces/)\w*", url)[0]
            except IndexError as err:
                raise ValueError("Input URL is not valid") from err
        return cls(space_id, format_str)

    @classmethod
    def from_user_tweets(cls, url: str, format_str: str):
        """Create a TwspaceDL object from the first space
        found in the 20 last user tweets"""
        user_id = TwspaceDL.user_id(url)
        headers = {
            "authorization": (
                "Bearer "
                "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
                "=1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
            ),
            "x-guest-token": TwspaceDL.guest_token(),
        }
        params = {
            "variables": (
                "{"
                f'"userId":"{user_id}",'
                '"count":20,'
                '"withTweetQuoteCount":true,'
                '"includePromotedContent":true,'
                '"withQuickPromoteEligibilityTweetFields":false,'
                '"withSuperFollowsUserFields":true,'
                '"withUserResults":true,'
                '"withNftAvatar":false,'
                '"withBirdwatchPivots":false,'
                '"withReactionsMetadata":false,'
                '"withReactionsPerspective":false,'
                '"withSuperFollowsTweetFields":true,'
                '"withVoice":true}'
            )
        }
        response = requests.get(
            "https://twitter.com/i/api/graphql/jpCmlX6UgnPEZJknGKbmZA/UserTweets",
            params=params,
            headers=headers,
        )
        tweets = response.text

        try:
            space_id = re.findall(r"(?<=https://twitter.com/i/spaces/)\w*", tweets)[0]
        except (IndexError, json.JSONDecodeError) as err:
            raise RuntimeError("User is not live") from err
        return cls(space_id, format_str)

    @classmethod
    def from_user_avatar(cls, user_url, format_str, auth_token):
        """Create a TwspaceDL object from a twitter user ongoing space"""
        headers = {
            "authorization": (
                "Bearer "
                "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
                "=1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
            ),
            "cookie": f"auth_token={auth_token};",
        }
        user_id = TwspaceDL.user_id(user_url)
        params = {"user_ids": user_id, "only_spaces": "true"}
        avatar_content = requests.get(
            f"https://twitter.com/i/api/fleets/v1/avatar_content",
            params=params,
            headers=headers,
        ).json()

        broadcast_id = avatar_content["users"][user_id]["spaces"]["live_content"][
            "audiospace"
        ]["broadcast_id"]
        return cls(broadcast_id, format_str)

    @staticmethod
    def user_id(user_url: str) -> str:
        """Get the id of a twitter using the url linking to their account"""
        screen_name = re.findall(r"(?<=twitter.com/)\w*", user_url)[0]

        params = {
            "variables": (
                "{"
                f'"screen_name":"{screen_name}",'
                '"withSafetyModeUserFields":true,'
                '"withSuperFollowsUserFields":true,'
                '"withNftAvatar":false'
                "}"
            )
        }
        headers = {
            "authorization": (
                "Bearer "
                "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
                "=1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
            ),
            "x-guest-token": TwspaceDL.guest_token(),
        }
        response = requests.get(
            "https://twitter.com/i/api/graphql/1CL-tn62bpc-zqeQrWm4Kw/UserByScreenName",
            headers=headers,
            params=params,
        )
        user_data = response.json()
        user_id = user_data["data"]["user"]["result"]["rest_id"]
        return user_id

    @staticmethod
    def guest_token() -> str:
        """Generate a guest token to authorize twitter api requests"""
        headers = {
            "authorization": (
                "Bearer "
                "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
                "=1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
            )
        }
        response = requests.post(
            "https://api.twitter.com/1.1/guest/activate.json", headers=headers
        ).json()
        guest_token = response["guest_token"]
        if not guest_token:
            raise RuntimeError("No guest token found after five retry")
        logging.debug(guest_token)
        return guest_token

    @cached_property
    def metadata(self) -> dict:
        """Get space metadata"""
        params = {
            "variables": (
                "{"
                f'"id":"{self.id}",'
                '"isMetatagsQuery":false,'
                '"withSuperFollowsUserFields":true,'
                '"withUserResults":true,'
                '"withBirdwatchPivots":false,'
                '"withReactionsMetadata":false,'
                '"withReactionsPerspective":false,'
                '"withSuperFollowsTweetFields":true,'
                '"withReplays":true,'
                '"withScheduledSpaces":true'
                "}"
            )
        }
        headers = {
            "authorization": (
                "Bearer "
                "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
                "=1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
            ),
            "x-guest-token": self.guest_token(),
        }
        response = requests.get(
            "https://twitter.com/i/api/graphql/jyQ0_DEMZHeoluCgHJ-U5Q/AudioSpaceById",
            params=params,
            headers=headers,
        )
        metadata = response.json()
        try:
            media_key = metadata["data"]["audioSpace"]["metadata"]["media_key"]
            logging.debug(media_key)
        except KeyError as error:
            logging.error(metadata)
            raise RuntimeError(metadata) from error
        return metadata

    @cached_property
    def filename(self) -> str:
        format_info = FormatInfo()
        format_info.set_info(self.metadata)
        filename = format_info.format(self.format_str)
        return filename

    @cached_property
    def dyn_url(self) -> str:
        metadata = self.metadata
        if (
            metadata["data"]["audioSpace"]["metadata"]["state"] == "Ended"
            and not metadata["data"]["audioSpace"]["metadata"][
                "is_space_available_for_replay"
            ]
        ):
            logging.error(
                (
                    "Can't Download. Space has ended, can't retrieve master url. "
                    "You can provide it with -f URL if you have it."
                )
            )
            raise ValueError("Space Ended")
        headers = {
            "authorization": (
                "Bearer "
                "AAAAAAAAAAAAAAAAAAAAANRILgAAAAAAnNwIzUejRCOuH5E6I8xnZz4puTs"
                "=1Zv7ttfk8LF81IUq16cHjhLTvJu4FA33AGWWjCpTnA"
            ),
            "cookie": "auth_token=",
        }
        media_key = metadata["data"]["audioSpace"]["metadata"]["media_key"]
        response = requests.get(
            "https://twitter.com/i/api/1.1/live_video_stream/status/" + media_key,
            headers=headers,
        )
        try:
            metadata = response.json()
        except Exception as err:
            raise RuntimeError("Space isn't available") from err
        dyn_url = metadata["source"]["location"]
        return dyn_url

    @cached_property
    def master_url(self) -> str:
        """Master URL for a space"""
        master_url = re.sub(
            r"(?<=/audio-space/).*", "master_playlist.m3u8", self.dyn_url
        )
        return master_url

    @property
    def playlist_url(self) -> str:
        """Get the URL containing the chunks filenames"""
        response = requests.get(self.master_url)
        playlist_suffix = response.text.splitlines()[3]
        domain = urlparse(self.master_url).netloc
        playlist_url = f"https://{domain}{playlist_suffix}"
        return playlist_url

    @property
    def playlist_text(self) -> str:
        """Modify the chunks URL using the master one to be able to download"""
        playlist_text = requests.get(self.playlist_url).text
        master_url_wo_file = re.sub("master_playlist\.m3u8.*", "", self.master_url)
        playlist_text = re.sub(r"(?=chunk)", master_url_wo_file, playlist_text)
        return playlist_text

    def write_playlist(self, save_dir: str = "./") -> None:
        """Write the modified playlist for external use"""
        filename = os.path.basename(self.filename) + ".m3u8"
        path = os.path.join(save_dir, filename)
        with open(path, "w", encoding="utf-8") as stream_io:
            stream_io.write(self.playlist_text)
        logging.info(f"{path} written to disk")

    def download(self) -> None:
        """Download a twitter space"""
        if not shutil.which("ffmpeg"):
            raise FileNotFoundError("ffmpeg not installed")
        metadata = self.metadata
        tempdir = self._tmpdir = tempfile.mkdtemp(dir=".")
        self.write_playlist(save_dir=tempdir)
        format_info = FormatInfo()
        format_info.set_info(metadata)
        state = metadata["data"]["audioSpace"]["metadata"]["state"]

        cmd_base = [
            "ffmpeg",
            "-y",
            "-stats",
            "-v",
            "warning",
            "-i",
            "-c",
            "copy",
            "-metadata",
            f"title={format_info['title']}",
            "-metadata",
            f"author={format_info['creator_name']}",
            "-metadata",
            f"episode_id={self.id}",
        ]

        filename = os.path.basename(self.filename)
        filename_m3u8 = os.path.join(tempdir, filename + ".m3u8")
        filename_old = os.path.join(tempdir, filename + ".m4a")
        cmd_old = cmd_base.copy()
        cmd_old.insert(1, "-protocol_whitelist")
        cmd_old.insert(2, "file,https,tls,tcp")
        cmd_old.insert(8, filename_m3u8)
        cmd_old.append(filename_old)

        if state == "Running":
            filename_new = os.path.join(tempdir, filename + "_new.m4a")
            cmd_new = cmd_base.copy()
            cmd_new.insert(6, (self.dyn_url))
            cmd_new.append(filename_new)

            concat_fn = os.path.join(tempdir, "list.txt")
            with open(concat_fn, "w", encoding="utf-8") as list_io:
                list_io.write(
                    "file "
                    + f"'{os.path.abspath(os.path.join(os.getcwd(), filename_old))}'"
                    + "\n"
                    + "file "
                    + f"'{os.path.abspath(os.path.join(os.getcwd(), filename_new))}'"
                )

            cmd_final = cmd_base.copy()
            cmd_final.insert(1, "-f")
            cmd_final.insert(2, "concat")
            cmd_final.insert(3, "-safe")
            cmd_final.insert(4, "0")
            cmd_final.insert(10, concat_fn)
            cmd_final.append(self.filename + ".m4a")

            try:
                subprocess.run(cmd_new, check=True)
                subprocess.run(cmd_old, check=True)
                subprocess.run(cmd_final, check=True)
            except subprocess.CalledProcessError as err:
                raise RuntimeError(" ".join(err.cmd)) from err
        else:
            try:
                subprocess.run(cmd_old, check=True)
            except subprocess.CalledProcessError as err:
                raise RuntimeError(" ".join(err.cmd)) from err
            shutil.move(filename_old, self.filename + ".m4a")

        logging.info("Finished downloading")



from .format_info import FormatInfo
from .twspace_dl import TwspaceDL
from .login import Login, load_from_file, write_to_file

__all__ = ["FormatInfo", "TwspaceDL", "Login", "load_from_file", "write_to_file"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['twspace_dl']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['twspace_dl = twspace_dl.__main__:main']}

setup_kwargs = {
    'name': 'twspace-dl',
    'version': '2022.1.16.0',
    'description': 'The only tool to record Twitter spaces (yet)',
    'long_description': '<!-- markdownlint-disable MD033 MD041 -->\n\n<div align="center">\n  <h1 id="twspace-dl">Twspace-dl</h1>\n  <p>\n    <a href="https://pypi.org/project/twspace-dl/">\n      <img src="https://img.shields.io/pypi/v/twspace-dl?style=for-the-badge" alt="PyPI">\n    </a>\n    <a href="https://pypi.org/project/twspace-dl/">\n      <img src="https://img.shields.io/pypi/dm/twspace-dl?label=DOWNLOADS%20%28PYPI%29&amp;style=for-the-badge" alt="PyPI DLs">\n    </a>\n    <a href="https://github.com/Ryu1845/twspace-dl/releases">\n      <img src="https://img.shields.io/github/downloads/Ryu1845/twspace-dl/total?label=DOWNLOADS%20%28GITHUB%29&amp;style=for-the-badge" alt="Github Releases DLs">\n    </a>\n  </p>\n  <p>A python module to download twitter spaces.</p>\n</div>\n\n## Requirements\n\nffmpeg\n\n## Install\n\n### From portable binaries\n\n[Linux](https://github.com/Ryu1845/twspace-dl/releases/latest/download/twspace_dl.bin)\n\n[Windows](https://github.com/Ryu1845/twspace-dl/releases/latest/download/twspace_dl.exe)\n\n### From PyPI\n\n```bash\npip install twspace-dl\n```\n\n### From source\n\n```bash\ngit clone --depth 1 https://github.com/Ryu1845/twspace-dl\ncd twspace-dl\npip install .\n```\n\n## Usage\n\n```bash\ntwspace_dl -i space_url\n```\n\n<details>\n<summary>With binaries</summary>\n\n### Windows\n\n```bash\n.\\twspace_dl.exe -i space_url\n```\n\n### Linux\n\n```bash\n./twspace_dl.bin -i space_url\n```\n\n</details>\n\n## Features\n\nHere\'s the output of the help option\n\n```txt\nusage: twspace_dl [-h] [-v] [-s] [-k] [--input-cookie-file COOKIE_FILE] [-i SPACE_URL | -U USER_URL] [-d DYN_URL] [-f URL] [-M PATH] [-o FORMAT_STR] [-m] [-p] [-u] [--write-url URL_OUTPUT] {login} ...\n\nScript designed to help download twitter spaces\n\npositional arguments:\n  {login}               (EXPERIMENTAL) Login to your account using username and password\n\noptions:\n  -h, --help            show this help message and exit\n  -v, --verbose\n  -s, --skip-download\n  -k, --keep-files\n  --input-cookie-file COOKIE_FILE\n\ninput:\n  -i SPACE_URL, --input-url SPACE_URL\n  -U USER_URL, --user-url USER_URL\n  -d DYN_URL, --from-dynamic-url DYN_URL\n                        use the dynamic url for the processes(useful for ended spaces) \n                        example: https://prod-fastly-ap-northeast-1.video.pscp.tv/Transcoding/v1/hls/\n                        zUUpEgiM0M18jCGxo2eSZs99p49hfyFQr1l4cdze-Sp4T-DQOMMoZpkbdyetgfwscfvvUkAdeF-I5hPI4bGoYg/\n                        non_transcode/ap-northeast-1/periscope-replay-direct-prod-ap-northeast-1-public/\n                        audio-space/dynamic_playlist.m3u8?type=live\n  -f URL, --from-master-url URL\n                        use the master url for the processes(useful for ended spaces) \n                        example: https://prod-fastly-ap-northeast-1.video.pscp.tv/Transcoding/v1/hls/\n                        YRSsw6_P5xUZHMualK5-ihvePR6o4QmoZVOBGicKvmkL_KB9IQYtxVqm3P_vpZ2HnFkoRfar4_uJOjqC8OCo5A/\n                        non_transcode/ap-northeast-1/periscope-replay-direct-prod-ap-\n                        northeast-1-public/audio-space/master_playlist.m3u8\n  -M PATH, --input-metadata PATH\n                        use a metadata json file instead of input url (useful for very old ended spaces)\n\noutput:\n  -o FORMAT_STR, --output FORMAT_STR\n  -m, --write-metadata  write the full metadata json to a file\n  -p, --write-playlist  write the m3u8 used to download the stream(e.g. if you want to use another downloader)\n  -u, --url             display the master url\n  --write-url URL_OUTPUT\n                        write master url to file\n```\n\n## Format\n\nYou can use the following identifiers for the formatting\n\n```python\n%(title)s\n%(id)s\n%(start_date)s\n%(creator_name)s\n%(creator_screen_name)s\n%(url)s\n```\n\nExample:\xa0`[%(creator_screen_name)s]-%(title)s|%(start_date)s`\n\n## Service\n\nThere\'s an example service file [there](https://github.com/Ryu1845/twspace-dl/blob/main/twspace-dl@.service)\nYou\'ll have to modify different stuff depending on how you installed or not the script.\n\nI\'d be very grateful if someone made a PR with a bash script automating those ;)\n\n### Cloned Repository\n\nIn that case, you\'ll have to modify the `WorkingDirectory` to where you cloned the repo.\n\n### Pip\n\n1. Change `WorkingDirectory` to where you want the spaces to be downloaded.\n2. Find where your executable is by running `which twspace_dl` in your terminal.\n3. Change the `/usr/bin/python twspace_dl` part to the path of your executable.\n\nNow to install the service, you can either install it as a user service(recommended if on your personal desktop), or as a normal service.\n\n### User\n\n1. Copy the file you modified earlier to `~/.config/systemd/user`(create the systemd directory if it doesn\'t exist).\n2. run\n\n``` bash\nsystemctl --user daemon-reload\nsystemctl --user start twspace-dl@USER_ID.service\n```\n\n`USER_ID` is the part after `https://twitter.com/` in the url of a twitter profile (i.e `https://twitter.com/USER_ID`)\n\nTo keep it working after restarts run:\n\n``` bash\nsystemctl --user enable twspace-dl@USER_ID.service\n```\n\n### System\n\n1. Copy the file you modified earlier to `/etc/systemd/system`.\n2. run\n\n``` bash\nsudo systemctl daemon-reload\nsudo systemctl start twspace-dl@USER_ID.service\n```\n\n`USER_ID` is the part after `https://twitter.com/` in the url of a twitter profile (i.e `https://twitter.com/USER_ID`)\n\nTo keep it working after restarts run:\n\n``` bash\nsudo systemctl enable twspace-dl@USER_ID.service\n```\n\n## Docker\n\n### Run once\n\n> Use ${pwd} in powershell, or $(pwd) in bash\n\n```bash\ndocker run --rm -v ${pwd}:/output ryu1845/twspace-dl -i space_url\n```\n\n### Run as monitoring service\n\nUsing a cookie can help solve some problem with the twitter api. However, using one is not necessary.\n\n#### Without cookie\n1. Download the `docker-compose.yml` and `.env` files and put them in a folder named `twspace-dl`.\n2. Edit `.env` and fill in the Twitter username you want to monitor.\n3. `docker-compose up -d`\n\n#### With cookies.txt\n1. Download the `docker-compose.yml`, `docker-compose.cookie.yml` and `.env` files and put them in a folder named `twspace-dl`.\n2. Edit `.env` and fill in the Twitter username you want to monitor and the path of your cookies.\n3. `docker-compose -f docker-compose.yml -f docker-compose.cookie.yml up -d`\n',
    'author': 'Ryu1845',
    'author_email': 'ryu@tpgjbo.xyz',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
